# Auxiliary routines used for testing and experimentation.
#

# Extract the main email message content.  Omit any attachments.
# Do this for the message with the given blobid.
#
proc content_only_from_blobid {blobid} {
  set full [::mailroom::db one {SELECT data FROM blob WHERE blobid=$blobid}]
  return [content_only $full]
}

proc content_only {full} {
  set m [mime::initialize -string $full]
  set out {}
  foreach {hdr content} [mime::getheader $m] {
    foreach x $content {
      append out "$hdr: $x\n"
    }
  }
  append out \n[::mailroom::message::_body $m]
  mime::finalize $m
  return $out
}

# Get the from name/address from the configuration
#
proc get_config_from {} {
  set fromaddr [::mailroom::db onecolumn {SELECT value FROM var WHERE name1='config' AND name2='fromaddr'}]
  set friendlyname [::mailroom::db onecolumn {SELECT value FROM var WHERE name1='config' AND name2='friendlyname'}]
  
  if {$fromaddr == ""} {
    return "N/A"
  } elseif {$friendlyname != {}} {
    return "$friendlyname <$fromaddr>"
  }
  
  return $fromaddr
}

# Set/update the main window title.
#
proc set_main_title {} {
  wm title . "[get_config_from] - $::mailroom::appTitle v$::mailroom::appVersion"
  wm iconname . $::mailroom::appTitle
}

proc number_format {number} {
    if {$number == ""} {
        return "0"
    } elseif {[string is integer -strict $number] == 0 && [string is double -strict $number] == 1} {
        set number [format "%.2f" $number]
    }

    set n $number
    while {[regsub {(\d)(\d\d\d)(?!\d)} $n {\1,\2} n]} {}
    return $n
}

proc bytes_to_kb {n} {
  if {$n == "" || $n == 0} { return 0 }

  set n [expr round($n / 1000)]
  return "[number_format $n]KB"
}

proc help_about {} {
  tk_messageBox -icon info -title "About $::mailroom::appTitle" \
    -message "About $::mailroom::appTitle" -detail "$::mailroom::appTitle v$::mailroom::appVersion"
}

proc escape_shell_arg {s} {
  if {$::tcl_platform(platform)=="windows"} {
    regsub -all -- {[^a-zA-Z0-9]} $s {^\0} s
  } else {
    regsub -all -- {[^a-zA-Z0-9]} $s {\\\0} s
  }
  return $s
}

proc view_file {filename} {
  global tcl_platform
  
  set filename [escape_shell_arg $filename]

  # It *is* generally a mistake to switch on $tcl_platform(os), particularly
  # in comparison to $tcl_platform(platform).  For now, let's just regard it
  # as a stylistic variation subject to debate.
  switch $tcl_platform(os) {
    Darwin {
      set command [list open $filename]
    }
    {Windows 95} -
    {Windows NT} {
      set command "[auto_execok start] {} [list $filename]"
    }
  }

  if [info exists command] {
    # Replace {*}$command by eval "$command" if you want < tcl 8.5 compatibility ([RA])
    # Added the '&' to launch the browser as background process. [Duoas]
    if [catch {exec {*}$command &} err] {
      tk_messageBox -icon error -message "error '$err' with '$command'"
    }
  } else {
    tk_messageBox -icon error -message \
      "$tcl_platform(os)/$tcl_platform(platform) is not yet ready for opening external files."  
  }
}

proc launch_external_browser {url} {
  global tcl_platform
  
  # It *is* generally a mistake to switch on $tcl_platform(os), particularly
  # in comparison to $tcl_platform(platform).  For now, let's just regard it
  # as a stylistic variation subject to debate.
  switch $tcl_platform(os) {
    Darwin {
      set command [list open]
    }
    HP-UX -
    Linux  -
    SunOS {
      foreach executable {firefox mozilla netscape iexplorer opera lynx
        w3m links epiphany galeon konqueror mosaic amaya browsex elinks} {
        set executable [auto_execok $executable]
        if [string length $executable] {
          # Do you want to mess with -remote?  How about other browsers?
          set command [list $executable $url &]
          break
        }
      }
    }
    {Windows 95} -
    {Windows NT} {
      set command "[auto_execok start] {}"
    }
  }

  if [info exists command] {
    # Replace {*}$command by eval "$command" if you want < tcl 8.5 compatibility ([RA])
    # Added the '&' to launch the browser as background process. [Duoas]
    if [catch {exec {*}$command [escape_shell_arg $url] &} err] {
      tk_messageBox -icon error -message "error '$err' with '$command'"
    }
  } else {
    tk_messageBox -icon error -message \
      "$tcl_platform(os)/$tcl_platform(platform) is not yet ready for browsing."
  }
}

namespace eval ::mailroom {
  proc julianday {{secs 0}} {
    if {$secs == 0} {
      set secs [clock seconds]
    }
    
    return [db eval {SELECT julianday($secs,'unixepoch')}]
  }

  proc full_from {} {
    set fromaddr [::mailroom::db onecolumn {SELECT value FROM var WHERE name1='config' AND name2='fromaddr'}]
    set friendlyname [::mailroom::db onecolumn {SELECT value FROM var WHERE name1='config' AND name2='friendlyname'}]
    
    if {$fromaddr == ""} {
      return "N/A"
    } elseif {$friendlyname != {}} {
      return "$friendlyname <$fromaddr>"
    }
    
    return $fromaddr
  }
}
